//
// Copyright (C) 2013-2016  Taiyou Sakashita, Hiroko Yamada
//
// Calculate Dailymean and Standard Deviation 2015/08/10
// Modified for using Julidan day 2016/07/01
//


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>
#include <time.h>
#include <math.h>
#include "bin.c"
#include "jday.c"


// Length of the Data in Column/Line Direction
int X;
int Y;


// Year/Month/Day/Minute
int YYYY;
int MM;
int DD;
int MN;

// UTC Year/Month/Day/Hour
int UYYYY;
int UMM;
int UDD;
int UHH;

// Number of Used Files
int nFile = 0;

// Number of Used Files for Standard Deviation
int nFile_std = 0;

// Error Hour Threshold
int Error_HH_THS = 7 + 3; // 11-17(7 hour is no data) plus 3 error hour

// Present Error Hour
int P_Error_HH;

// Dailymean
double *DM;

// Standard Deviation
double *STD;

// File Type
char ftype[512];

// Directory Name to Read Original Data
char input_DIR[512];

// Directory Name to Write Dailymean Data
char DayMn_DIR[512];
// TIME DIFF 
double TIME_DIFF;
char TIME_Code[512];

// Julian day local 
double jd_local = 0;
double jd_std_local = 0;
int FLG_DAILY = 0;
int FLG_STD = 1;


void cal_dailymean();
void cal_hourlymean(int PHH);
void cal_std(char *input_DIR);
void add_data_d(float *DR, double *ADD);
void add_data_f(float *DR, float *ADD);
void cal_mean_f(float *DD, int N, float *RR);
void cal_mean_d(double *DD, int N, double *RR);
void cal_var(float *DR);
void standard_deviation();
int get_lastday(int Year, int Month);
void date_errorcheck();
void initialization();
void read_formatfile();
void shift_time(int Flag, int *Shift_Time);



int main(int argc, char *argv[])
{


  int I;

  // Filename to Write Dailymean Data
  char wfilename[512];

  // Filename to Write Standard Deviation Data
  char std_filename[512];

  // Year/Month/Day/Minute
  YYYY = atoi(argv[1]);
  MM   = atoi(argv[2]);
  DD   = atoi(argv[3]);

//check & make output/MONTH directory
  struct stat  st;
  char fname[20];
  char str1[20] = "output/";
  sprintf(fname,"%s%d%02d",str1,YYYY,MM);

  if (stat(fname, &st) != 0){
       printf("%s dosen't exist\n", fname);
       if(mkdir(fname, 0777)==0){
             printf("Succeeded to create a directory\n");
       }else{
                printf("Failed to create a directory\n");
            }
       }
       else {
            switch (st.st_mode & S_IFMT) {
                   case S_IFREG:
                        printf("%s is a file\n", fname);
                        break;
                   case S_IFDIR:
                        printf("%s is a dir\n", fname);
                        break;
                   default:
                        printf("%s isn't a file or dir\n", fname);
                        break;
             }
        }


  date_errorcheck();
  initialization();

  double jd_jul_12ji=(double)ymd2jd(YYYY,MM,DD);
  jd_local= jd_jul_12ji - (double)12/24-(double)TIME_DIFF/24;
  jd_std_local= jd_local;
        
  // Calculate Dailymean
  cal_dailymean();

  // Filename cut 
  char ftype_cut[]="";
  strncpy(ftype_cut, ftype, strlen(ftype)-4);
  *(ftype_cut+ strlen(ftype)-4) = 0;              /* string end null */
  
  sprintf(wfilename, "%s%d%02d/%d%02d%02d.%s%s.dm.bin", DayMn_DIR, YYYY, MM, YYYY, MM, DD, TIME_Code, ftype_cut);

  // Write Dailymean Data
  if(nFile > 0) 
  {
     // Filename cut 
     char ftype_cut[]="";
     strncpy(ftype_cut, ftype, strlen(ftype)-4);
     *(ftype_cut+ strlen(ftype)-4) = 0;              /* string end null */
     d2bin(wfilename, X, Y, DM);

     // Calculate Standard Deviation
     cal_std(input_DIR);

     sprintf(std_filename, "%s%d%02d/%d%02d%02d.%s%s.std.bin", DayMn_DIR, YYYY, MM, YYYY, MM, DD, TIME_Code, ftype_cut);

     // Write Standard Deviation Data
     if(nFile_std > 0) d2bin(std_filename, X, Y, STD);

  }//fi


  free(DM);
  free(STD);


  return 0;


}//main



void cal_dailymean()
{


  int I;

  // Filename of Total Data
  char tfilename[512];


  P_Error_HH = 0;

  // Calculate Sum of Hourly Data
  for(I=0;I<=23;I++)
  {
     cal_hourlymean(I);

  }//I

  if(P_Error_HH >= Error_HH_THS)
  {
     printf("Loss of data is too much on %d%02d%02d\n", YYYY, MM, DD);
     exit(1);

  }//fi


  // Calculate Dailymean
  if(nFile > 0) cal_mean_d(DM, 24, DM);

  return;

}//cal_dailymean


void cal_hourlymean(int PHH)
{

  int I;

  // Number of Data of 1 Hour
  int nFile_HH;

  // UTC Time
  int UTime[5+1];

  // Data Read
  float *DR;

  // Hourl Mean
  float *HM;

  // Filename of Original Data
  char filename[512];

  struct stat st;


  if((DR = (float *)malloc(sizeof(float)*((X+1)*(Y+1)))) == NULL) {printf("malloc failed."); exit(1);}
  if((HM = (float *)malloc(sizeof(float)*((X+1)*(Y+1)))) == NULL) {printf("malloc failed."); exit(1);}

  shift_time(FLG_DAILY, UTime);
  UYYYY = UTime[1];
  UMM = UTime[2];
  UDD = UTime[3];
  UHH = UTime[4];

  nFile_HH = 0;

  for(MN=0;MN<=59;MN++)
  {
     sprintf(filename, "%s%d%02d/%d%02d%02d%02d%02d%s", input_DIR, UYYYY, UMM, UYYYY, UMM, UDD, UHH, MN, ftype);

     if(stat(filename, &st) == 0)
     {
        bin2f(filename, X, Y, DR);
        add_data_f(DR, HM);
        nFile_HH++;

     }//fi
  }//MN

  if(nFile_HH > 0)
  {
     cal_mean_f(HM, nFile_HH, HM);
     nFile++;
     add_data_d(HM, DM);

  }//fi


  if(nFile_HH == 0)
  {
     P_Error_HH++;
     printf("Data of %d%02d%02d%02d is not found\n", UYYYY, UMM, UDD, UHH);

  }//fi

  free(DR);
  free(HM);


  return;


}//cal_hourlymean


void cal_std(char *input_DIR)
{


  int I,J;

  // Data Read
  float *DR;

  // Filename of Original Data
  char filename[512];

  // UTC Time
  int UTime[5+1];

  struct stat st;


  if((DR = (float *)malloc(sizeof(float)*((X+1)*(Y+1)))) == NULL) {printf("malloc failed."); exit(1);}


  // Calculate Variance
  for(I=0;I<=23;I++)
  {
     shift_time(FLG_STD, UTime);
     UYYYY = UTime[1];
     UMM = UTime[2];
     UDD = UTime[3];
     UHH = UTime[4];


     for(MN=0;MN<=59;MN++)
     {
        sprintf(filename, "%s%d%02d/%d%02d%02d%02d%02d%s", input_DIR, UYYYY, UMM, UYYYY, UMM, UDD, UHH, MN, ftype);

        if(stat(filename, &st) == 0)
        {
           bin2f(filename, X, Y, DR);
           cal_var(DR);
           nFile_std++;

        }//fi
     }//MN
  }//I

  // Calculate Standard Deviation
  if(nFile_std > 0)
  {
     cal_mean_d(STD, nFile_std, STD);
     standard_deviation();

  }//fi


  free(DR);


  return;


}//cal_std


void add_data_d(float *DR, double *ADD)
{

  int I,J;

  for(I=1;I<=Y;I++)
  {
     for(J=1;J<=X;J++)
     {
        ADD[(I-1)*X+J] = ADD[(I-1)*X+J] + DR[(I-1)*X+J];

     }//J
  }//I


  return;


}//add_data_d


void add_data_f(float *DR, float *ADD)
{

  int I,J;

  for(I=1;I<=Y;I++)
  {
     for(J=1;J<=X;J++)
     {
        ADD[(I-1)*X+J] = ADD[(I-1)*X+J] + DR[(I-1)*X+J];

     }//J
  }//I


  return;


}//add_data_f



void cal_mean_f(float *DD, int N, float *RR)
{

  int I,J;


  for(I=1;I<=Y;I++)
  {
     for(J=1;J<=X;J++)
     {
        RR[(I-1)*X+J] = DD[(I-1)*X+J] / N;

     }//J
  }//I


  return;


}//cal_mean_f


void cal_mean_d(double *DD, int N, double *RR)
{

  int I,J;


  for(I=1;I<=Y;I++)
  {
     for(J=1;J<=X;J++)
     {
        RR[(I-1)*X+J] = DD[(I-1)*X+J] / N;

     }//J
  }//I


  return;


}//cal_mean_d


void cal_var(float *DR)
{

  int I,J;


  for(I=1;I<=Y;I++)
  {
     for(J=1;J<=X;J++)
     {
        STD[(I-1)*X+J] = STD[(I-1)*X+J] + (DR[(I-1)*X+J] - DM[(I-1)*X+J]) * (DR[(I-1)*X+J] - DM[(I-1)*X+J]);

     }//J
  }//I


  return;


}//cal_var


void standard_deviation()
{

  int I,J;


  for(I=1;I<=Y;I++)
  {
     for(J=1;J<=X;J++)
     {
        STD[(I-1)*X+J] = sqrt(STD[(I-1)*X+J]);

     }//J
  }//I


  return;


}//standard_deviation


int get_lastday(int Year, int Month)
{

  // Last Day of the Month
  int Last_DD;


  time_t timer;
  struct tm base;
  struct tm *shift;

  base.tm_year = Year-1900;
  base.tm_mon  = Month;
  base.tm_mday = 1;
  base.tm_hour = 0-1;
  base.tm_min  = 0;
  base.tm_sec  = 0;

  timer = mktime(&base);

  shift = localtime(&timer);

  Last_DD = shift->tm_mday;


  return Last_DD;


}//get_lastday


void date_errorcheck()
{

  // Last Day of the Month
  int Last_DD;


  // Year Check
  if(YYYY <= 0)
  {
     printf("Year %d is less than or equal to 0\n", YYYY);
     exit(1);

  }//fi

  // Month Check
  if(MM <= 0 || MM > 12)
  {
     printf("Month %d does not exist\n", MM);
     exit(1);

  }//fi

  // Day Check
  Last_DD = get_lastday(YYYY, MM);

  if(DD > Last_DD || DD <= 0)
  {
     printf("Date of %d%02d%02d does not exist\n", YYYY, MM, DD);
     exit(1);

  }//fi


  return;


}//date_errorcheck


void shift_time(int flg, int *Shift_Time)
{
        double jd=0;

        if(flg == FLG_DAILY){
                jd_local= jd_local+(double)1/24; // Julian local day
                jd = jd_local;
         }
        else if(flg == FLG_STD){
                jd_std_local= jd_std_local+(double)1/24; // Julian local day
                jd = jd_std_local;
        }
        
        jd2ymdhn(jd,Shift_Time);

  return;

}//shift_time

void initialization()
{

  int I,J;


  read_formatfile();

  if((DM = (double *)malloc(sizeof(double)*((X+1)*(Y+1)))) == NULL) {printf("malloc failed."); exit(1);}
  if((STD = (double *)malloc(sizeof(double)*((X+1)*(Y+1)))) == NULL) {printf("malloc failed."); exit(1);}

  // Array Initialization
  for(I=1;I<=Y;I++)
  {
     for(J=1;J<=X;J++)
     {
        DM[(I-1)*X+J] = 0;
        STD[(I-1)*X+J] = 0;

     }//J
  }//I


  return;


}//initialization


void read_formatfile()
{

  int I;

  FILE *fp;

  // Present Line
  int pline = 1;

  char filename[512];
  char readline[1000];
  char *param[100+1];


  sprintf(filename, "config/format.txt");

  // Read Format File
  for(I=0;I<=100;I++)
  {
     if((param[I] = (char *)malloc(sizeof(char)*1000)) == NULL) {printf("malloc failed.\n"); exit(1);}

  }//I

  printf("%s  Loading ...", filename);

  fp = fopen(filename, "r");

  if(fp == NULL)
  {

     printf("%s read error\n", filename);
     exit(1);

  }//fi

  while( fgets(readline, 1000, fp) != NULL)
  {
     strcpy(param[pline], strtok(readline, ", "));

     pline++;

  }//while

  printf(" finish\n");

  fclose(fp);


  TIME_DIFF = atof(param[1]);   // int->doble for India 5.5H

  sprintf(TIME_Code, "%s", param[2]);

  X = atoi(param[3]);
  Y = atoi(param[4]);

  sprintf(ftype, "%s", param[5]);

  sprintf(input_DIR, "%s", param[6]);
  sprintf(DayMn_DIR, "%s", param[7]);


  for(I=0;I<=100;I++) free(param[I]);


  return;


}//read_formatfile

